/* 
 * Protocol Families.
 *
 */
#define HAVE_AFUNIX 1
#define HAVE_AFINET 1
#define HAVE_AFAX25 0
#define HAVE_AFIPX 1

/*
 * Device Hardware types.
 * 
 */
#define HAVE_HWETHER 1
#define HAVE_HWSLIP 1
#define HAVE_HWAX25 0
#define HAVE_HWPPP 1
